@@ -26,6 +26,7 @@ from dask.bag.core import (
     inline_singleton_lists,
     optimize,
     from_delayed,
+    total_mem_usage,
 )
 from dask.bag.utils import assert_eq
 from dask.delayed import Delayed
@@ -647,7 +648,7 @@ def test_read_text_large():
 def test_read_text_encoding():
     with tmpfile() as fn:
         with open(fn, "wb") as f:
-            f.write((u"你好！" + os.linesep).encode("gb18030") * 100)
+            f.write(("你好！" + os.linesep).encode("gb18030") * 100)
         b = db.read_text(fn, blocksize=100, encoding="gb18030")
         c = db.read_text(fn, encoding="gb18030")
         assert len(b.dask) > 5
@@ -683,11 +684,11 @@ def test_from_s3():
     pytest.importorskip("s3fs")
 
     five_tips = (
-        u"total_bill,tip,sex,smoker,day,time,size\n",
-        u"16.99,1.01,Female,No,Sun,Dinner,2\n",
-        u"10.34,1.66,Male,No,Sun,Dinner,3\n",
-        u"21.01,3.5,Male,No,Sun,Dinner,3\n",
-        u"23.68,3.31,Male,No,Sun,Dinner,2\n",
+        "total_bill,tip,sex,smoker,day,time,size\n",
+        "16.99,1.01,Female,No,Sun,Dinner,2\n",
+        "10.34,1.66,Male,No,Sun,Dinner,3\n",
+        "21.01,3.5,Male,No,Sun,Dinner,3\n",
+        "23.68,3.31,Male,No,Sun,Dinner,2\n",
     )
 
     # test compressed data
@@ -947,7 +948,7 @@ def test_to_textfiles_name_function_warn():
 
 
 def test_to_textfiles_encoding():
-    b = db.from_sequence([u"汽车", u"苹果", u"天气"], npartitions=2)
+    b = db.from_sequence(["汽车", "苹果", "天气"], npartitions=2)
     for ext, myopen in ext_open:
         with tmpdir() as dir:
             c = b.to_textfiles(
@@ -960,7 +961,7 @@ def test_to_textfiles_encoding():
             text = f.read()
             if hasattr(text, "decode"):
                 text = text.decode("gb18030")
-            assert u"天气" in text
+            assert "天气" in text
             f.close()
 
 
@@ -1011,12 +1012,12 @@ def test_string_namespace():
 
 
 def test_string_namespace_with_unicode():
-    b = db.from_sequence([u"Alice Smith", u"Bob Jones", "Charlie Smith"], npartitions=2)
+    b = db.from_sequence(["Alice Smith", "Bob Jones", "Charlie Smith"], npartitions=2)
     assert list(b.str.lower()) == ["alice smith", "bob jones", "charlie smith"]
 
 
 def test_str_empty_split():
-    b = db.from_sequence([u"Alice Smith", u"Bob Jones", "Charlie Smith"], npartitions=2)
+    b = db.from_sequence(["Alice Smith", "Bob Jones", "Charlie Smith"], npartitions=2)
     assert list(b.str.split()) == [
         ["Alice", "Smith"],
         ["Bob", "Jones"],
@@ -1065,7 +1066,7 @@ def test_bag_class_extend():
 
 
 def test_gh715():
-    bin_data = u"\u20ac".encode("utf-8")
+    bin_data = "\u20ac".encode("utf-8")
     with tmpfile() as fn:
         with open(fn, "wb") as f:
             f.write(bin_data)
@@ -1164,16 +1165,52 @@ def test_zip(npartitions, hi=1000):
 
 @pytest.mark.parametrize("nin", [1, 2, 7, 11, 23])
 @pytest.mark.parametrize("nout", [1, 2, 5, 12, 23])
-def test_repartition(nin, nout):
+def test_repartition_npartitions(nin, nout):
     b = db.from_sequence(range(100), npartitions=nin)
     c = b.repartition(npartitions=nout)
-
     assert c.npartitions == nout
     assert_eq(b, c)
     results = dask.get(c.dask, c.__dask_keys__())
     assert all(results)
 
 
+@pytest.mark.parametrize(
+    "nin, nout",
+    [(1, 1), (2, 1), (5, 1), (1, 2), (2, 2), (5, 2), (1, 5), (2, 5), (5, 5),],
+)
+def test_repartition_partition_size(nin, nout):
+    b = db.from_sequence(range(1, 100), npartitions=nin)
+    total_mem = sum(b.map_partitions(total_mem_usage).compute())
+    c = b.repartition(partition_size=(total_mem // nout))
+    assert c.npartitions >= nout
+    assert_eq(b, c)
+
+
+def test_multiple_repartition_partition_size():
+    b = db.from_sequence(range(1, 100), npartitions=1)
+    total_mem = sum(b.map_partitions(total_mem_usage).compute())
+
+    c = b.repartition(partition_size=(total_mem // 2))
+    assert c.npartitions >= 2
+    assert_eq(b, c)
+
+    d = c.repartition(partition_size=(total_mem // 5))
+    assert d.npartitions >= 5
+    assert_eq(c, d)
+
+
+def test_repartition_partition_size_complex_dtypes():
+    np = pytest.importorskip("numpy")
+
+    b = db.from_sequence([np.array(range(100)) for _ in range(4)], npartitions=1)
+    total_mem = sum(b.map_partitions(total_mem_usage).compute())
+
+    new_partition_size = total_mem // 4
+    c = b.repartition(partition_size=new_partition_size)
+    assert c.npartitions >= 4
+    assert_eq(b, c)
+
+
 def test_repartition_names():
     b = db.from_sequence(range(100), npartitions=5)
     c = b.repartition(2)
@@ -1187,6 +1224,12 @@ def test_repartition_names():
     assert b is c
 
 
+def test_repartition_input_errors():
+    with pytest.raises(ValueError):
+        bag = db.from_sequence(range(10))
+        bag.repartition(npartitions=5, partition_size="5MiB")
+
+
 def test_accumulate():
     parts = [[1, 2, 3], [4, 5], [], [6, 7]]
     dsk = dict((("test", i), p) for (i, p) in enumerate(parts))
